#######################
# auxiliary functions #
#######################

def load_vector(V, file_name):
  n = V.degree()
  v = V.zero_vector()
  f = open(file_name)

  for i in xrange(n):
    l = f.readline().replace('\n','')
    v[i] = QQ(l)

  f.close()
  return v
#

def save_vector(v, file_name):
  f = open(file_name, 'w')

  for i in xrange(len(v)):
    f.write(str(v[i]))
    f.write('\n')

  f.close()
#

def load_vector_list(V, file_name, size):
  n = V.degree()
  V = VectorSpace(QQ, n)
  L = []
  f = open(file_name)

  for i in xrange(size):
    v = V.zero_vector()
    for j in xrange(n):
      l = f.readline().replace('\n','')
      v[j] = QQ(l)
    L.append(v)
    f.readline()
    
  f.close() 
  return L
#


def verify_solution():
  # semidefinite method initialization
  sigma_count = 8 #number of sigma-types

  #a_i := size of F_i
  a = [5, 95, 47, 191, 135, 95, 101, 148]

  #r_i := number of vectors v_j such that A_i = w_j * v_j x (v_j)^T
  r = [3, 45, 21, 63, 60, 43, 31, 25]

  # index of sigma in flags.rat (from flagmatic 1.5.1)
  flagmatic_id = [2,3,4,7,10,14,13,9]
  sigma_id = {}  # inverse of ^^
  for i in xrange(sigma_count): sigma_id[flagmatic_id[i]]=i

  vector_size = 8157 #number of 7-vertex 3-graphs without K4-
  V = VectorSpace(QQ, vector_size)
  
  print "Loading 7-vertex densities in the extremal"
  extremal7 = load_vector(V, 'extremal-7')

  print "Loading inequalities (4) and (5) generated from kappa_1 and kappa_2"
  ineq1=load_vector(V, 'ineq1')
  ineq2=load_vector(V, 'ineq2')

  print "Loading coefficients gamma_1 and gamma_2"
  f=open('gamma1')
  gamma1=QQ(f.readline())
  f.close()
  f=open('gamma2')
  gamma2=QQ(f.readline())
  f.close()


  vectors = [None]*sigma_count
  weights = [None]*sigma_count
  for sigma in xrange(sigma_count):
    print "Loading matrix A_%d (vectors v^j_%d and weights w^j_%d)" %(sigma+1, sigma+1, sigma+1)
    vectors[sigma] = load_vector_list(VectorSpace(QQ,a[sigma]), 'vectors_sigma%d'%(sigma+1), r[sigma])
    weights[sigma] = load_vector(VectorSpace(QQ,r[sigma]), 'weights_sigma%d'%(sigma+1))


  print "\nComputing the final vector..."
  final_vector = gamma1*ineq1 + gamma2*ineq2
  f=open('flags.rat')

  c=-1
  for l in f:
    #format: to the value of v[i] sigma-type flags a*b contributes with value A_{a,b}*num/denum
    (i,sigma_flagmatic,a,b,num,denum) = map(int,l.split())
    i-=1 # flagmatic counts from 1 instead of 0
    a-=1 # flagmatic counts from 1 instead of 0
    b-=1 # flagmatic counts from 1 instead of 0

    if (c!=i) or (sigma!=sigma_id[sigma_flagmatic]):
      if (c!=-1):
        for j in xrange(r[sigma]): final_vector[c]+=y[j]*weights[sigma][j]
      sigma=sigma_id[sigma_flagmatic]
      y=[0]*r[sigma]

    if (c!=i):
      if (c%100==0): print "progress: %d/%d" % (c,vector_size)
      c=i

    for j in xrange(r[sigma]):
      y[j] += vectors[sigma][j][a] * vectors[sigma][j][b] * (1+(a!=b)) * num/denum

  f.close()
  for j in xrange(r[sigma]): final_vector[c]+=y[j]*weights[sigma][j]
  print "-> DONE\n"
  print "Checking if the final vector is the one promised in Lemma 5..."
  
  for i in xrange(vector_size):
    if (extremal7[i]>0):
      if (final_vector[i]!=0): print "-> NO"; return -1
    else:
      if (final_vector[i]>=0): print "-> NO"; return -1
  print "-> YES\n"

  return final_vector
#



final_vector = verify_solution()
print "Saving the final vector to 'final_vector'"
save_vector(final_vector, 'final_vector')
